require 'studio_game/player'
require 'studio_game/treasure_trove'
require 'pry'
module StudioGame
  describe Player do # rubocop:disable Metrics/BlockLength
    let(:player) { Player.new('jose', 150) }
    let(:hammer) { Treasure.new(:hammer, 50) }
    let(:crowbar) { Treasure.new(:crowbar, 400) }
    before do
      $stdout = StringIO.new
    end

    context 'validations' do # rubocop:disable Metrics/BlockLength
      before do
        @initial_health = player.health
      end
      it 'has a capitalize name' do
        expect(player.name).to eq('Jose')
      end

      it 'has a initial health' do
        expect(player.health).to eq(150)
      end

      it 'has a initial score' do
        expect(player.score).to eq(@initial_health + player.points)
      end

      it 'has a string representation' do
        player.found_treasure(Treasure.new(:hammer, 50))
        player.found_treasure(Treasure.new(:hammer, 50))
        expect(player.to_s).to eq(
          "I'm Jose with health = 150, points = 100, and score = 250."
        )
      end

      it 'increases health by 15 when w00ted' do
        player.w00t
        expect(player.health).to be(@initial_health + 15)
      end

      it 'decreases health by 10 when blammed' do
        player.blam
        expect(player.health).to be(@initial_health - 10)
      end

      it 'computes a score as the sum of its health and points' do
        player.found_treasure(Treasure.new(:hammer, 50))
        player.found_treasure(Treasure.new(:hammer, 50))

        expect(player.score).to eq(250)
      end

      it 'computes points as the sum of all treasure points' do
        expect(player.points).to eq(0)

        expect(player.found_treasure(hammer)).to eq(Treasure.new(:hammer, 50))

        expect(player.points).to eq(50)

        expect(player.found_treasure(crowbar)).to eq(Treasure.new(:crowbar, 400))

        expect(player.points).to eq(450)

        expect(player.found_treasure(hammer)).to eq(Treasure.new(:hammer, 50))

        expect(player.points).to eq(500)
      end

      it "assigns a treasure for points during a player's turn" do
        game = Game.new('Knuckleheads')
        player = Player.new('moe')

        game.add_player(player)

        game.play(1)

        expect(player.points).not_to be_zero
      end
    end

    context 'when is created witout initial health' do
      let(:player) { Player.new('jose') }

      it 'has a initial health of 100' do
        expect(player.health).to eq(100)
      end
    end

    context 'when is strong' do
      let(:player) { Player.new('jose', 167) }

      it 'has a health above 150' do
        expect(player.strong?).to eq(true)
      end

      it 'has a response for continuing playing' do
        expect(player.status).to eq("#{player.name} is still standing.")
      end
    end

    context 'when is weak' do
      let(:player) { Player.new('jose', 49) }

      it 'has a health under 150' do
        expect(player.strong?).to eq(false)
      end

      it 'has a response for player is out' do
        expect(player.status).to eq("#{player.name} is out blammed.")
      end
    end

    context 'when has various treasure' do
      it 'yields each found treasure and its total points' do
        player.found_treasure(Treasure.new(:skillet, 100))
        player.found_treasure(Treasure.new(:skillet, 100))
        player.found_treasure(Treasure.new(:hammer, 50))
        player.found_treasure(Treasure.new(:bottle, 5))
        player.found_treasure(Treasure.new(:bottle, 5))
        player.found_treasure(Treasure.new(:bottle, 5))
        player.found_treasure(Treasure.new(:bottle, 5))
        player.found_treasure(Treasure.new(:bottle, 5))

        yielded = []
        player.each_found_treasure do |treasure|
          yielded << treasure
        end

        expect(yielded).to eq([
                                Treasure.new(:skillet, 200),
                                Treasure.new(:hammer, 50),
                                Treasure.new(:bottle, 25)
                              ])
      end
    end

    context 'when is loaded by a csv file' do
      let(:player) { Player.from_csv('larry,150') }
      it 'can be created ' do
        expect(player.name).to eq('Larry')
        expect(player.health).to eq(150)
      end
    end
  end
end
